@@ -1,18 +1,19 @@
 # coding: utf-8
+from functools import partial
+from hashlib import md5
 import random
 import time
 import urllib
-import re
-from hashlib import md5
-from functools import partial
-try:
-    import json
-except ImportError:
-    import simplejson as json
+import warnings
+
 from vkontakte import http
+try:
+    import simplejson as json
+except ImportError:
+    import json
 
 API_URL = 'http://api.vk.com/api.php'
-SECURE_API_URL = 'https://api.vkontakte.ru/method/'
+SECURE_API_URL = 'https://api.vk.com/method/'
 DEFAULT_TIMEOUT = 1
 REQUEST_ENCODING = 'utf8'
 
@@ -21,14 +22,15 @@ REQUEST_ENCODING = 'utf8'
 # http://vk.com/developers.php?o=-1&p=%D0%A0%D0%B0%D1%81%D1%88%D0%B8%D1%80%D0%B5%D0%BD%D0%BD%D1%8B%D0%B5_%D0%BC%D0%B5%D1%82%D0%BE%D0%B4%D1%8B_API&s=0
 # http://vk.com/developers.php?o=-1&p=%D0%9E%D0%BF%D0%B8%D1%81%D0%B0%D0%BD%D0%B8%D0%B5_%D0%BC%D0%B5%D1%82%D0%BE%D0%B4%D0%BE%D0%B2_API&s=0
 COMPLEX_METHODS = ['secure', 'ads', 'messages', 'likes', 'friends',
-    'groups', 'photos', 'wall', 'newsfeed', 'notifications', 'audio',
-    'video', 'docs', 'places', 'storage', 'notes', 'pages',
-    'activity', 'offers', 'questions', 'subscriptions',
-    'users', 'status', 'polls', 'account', 'auth', 'stats']
+                   'groups', 'photos', 'wall', 'newsfeed', 'notifications', 'audio',
+                   'video', 'docs', 'places', 'storage', 'notes', 'pages',
+                   'activity', 'offers', 'questions', 'subscriptions',
+                   'users', 'status', 'polls', 'account', 'auth', 'stats', 'database']
 
 
 class VKError(Exception):
     __slots__ = ["error"]
+
     def __init__(self, error_data):
         self.error = error_data
         Exception.__init__(self, str(self))
@@ -45,9 +47,14 @@ class VKError(Exception):
     def params(self):
         return self.error['request_params']
 
+    @property
+    def redirect_uri(self):
+        return self.error['redirect_uri']
+
     def __str__(self):
         return "Error(code = '%s', description = '%s', params = '%s')" % (self.code, self.description, self.params)
 
+
 def _encode(s):
     if isinstance(s, (dict, list, tuple)):
         s = json.dumps(s, ensure_ascii=False, encoding=REQUEST_ENCODING)
@@ -55,7 +62,16 @@ def _encode(s):
     if isinstance(s, unicode):
         s = s.encode(REQUEST_ENCODING)
 
-    return s # this can be number, etc.
+    return s  # this can be number, etc.
+
+
+def _json_iterparse(response):
+    response = response.strip().decode('utf8', 'ignore').encode('utf8')
+    decoder = json.JSONDecoder(encoding="utf8", strict=False)
+    idx = 0
+    while idx < len(response):
+        obj, idx = decoder.raw_decode(response, idx)
+        yield obj
 
 
 def signature(api_secret, params):
@@ -65,13 +81,15 @@ def signature(api_secret, params):
 
 # We have to support this:
 #
-#   >>> vk = API(key, secret)
-#   >>> vk.get('getServerTime')  # "get" is a method of API class
-#   >>> vk.friends.get(uid=123)  # "get" is a part of vkontakte method name
+# >>> vk = API(key, secret)
+# >>> vk.get('getServerTime')  # "get" is a method of API class
+# >>> vk.friends.get(uid=123)  # "get" is a part of vkontakte method name
 #
 # It works this way: API class has 'get' method but _API class doesn't.
 
+
 class _API(object):
+
     def __init__(self, api_id=None, api_secret=None, token=None, **defaults):
 
         if not (api_id and api_secret or token):
@@ -92,10 +110,17 @@ class _API(object):
                 'request_params': kwargs,
             })
 
-        data = json.loads(response, strict=False)
-        if "error" in data:
-            raise VKError(data["error"])
-        return data['response']
+        # there may be a response after errors
+        errors = []
+        for data in _json_iterparse(response):
+            if "error" in data:
+                errors.append(data["error"])
+            if "response" in data:
+                for error in errors:
+                    warnings.warn("%s" % error)
+                return data["response"]
+
+        raise VKError(errors[0])
 
     def __getattr__(self, name):
         '''
@@ -124,7 +149,7 @@ class _API(object):
             kwargs[key] = _encode(value)
 
         if self.token:
-            # http://vkontakte.ru/developers.php?oid=-1&p=Выполнение_запросов_к_API
+            # https://vk.com/dev/api_requests
             params = dict(
                 access_token=self.token,
             )
